# Copyright: Immanuel Scholz
# License: LGPL
require "rubygame"

module Rubygame

  attr_reader :queue

  Rubygame.init
  TTF.setup
  $font = TTF.new "FreeSans.ttf", 12
  $queue = EventQueue.new()
  $clock = Clock.new()
  $registered_events = []
  $tick_events = []

  # Register an tick event handler (called on every tick)
  def register_on_tick &block
    $tick_events << block
  end

  # Register an event handler
  # event:: An hash with the conditions for the handler to be called. Multiple 
  #         conditions are combined with AND. Possible values are:
  #       :key:: The key of an key event (class defaults to KeyDownEvent)
  #       :class:: The event is of the given class
  #       :button:: The mouse button identifier of a mouse event
  # block:: The given block is executed, when the conditions are fullified.
  #         First parameter is the event object of the event that caused the handler
  #         to be called
  def register event, &block
    # setting some sane default classes
    event[:class] = KeyDownEvent if event[:key] and not event[:class]
    event[:class] = MouseDownEvent if event[:button] and not event[:class]
    # constructing test code for the event
    code = Proc.new do |e|
      (event[:class].nil? or e.is_a? event[:class]) and
      (not e.respond_to? :key or event[:key].nil? or 
          (event[:key].is_a?(Regexp) ? Key::KEY2ASCII[e.key] =~ event[:key] : 
                                       e.key == event[:key])) and
      (not e.respond_to? :button or event[:button].nil? or e.button == event[:button])
    end
    $registered_events << [code, block]
  end

  def start
    catch :quit do
      loop do
        $clock.tick()
        $tick_events.each{|e| e[nil]}
        $queue.each do |event|
          throw :quit if event.is_a? QuitEvent
          $registered_events.each do |code, block|
            block[event] if code[event]
          end
          yield event if block_given?
        end
        $backbuffer.blit($screen, [0,0]) if $backbuffer and $screen
        $screen.update() if $screen
      end
    end
    Rubygame.quit
  end


  module Matrix2d
    def ysize
      size
    end
    
    def xsize
      at(0).size
    end

    def [] x,y = nil
      y.nil? ? at(x) : at(y)[x]
    end
    
    def []= x,y,v
      at(y)[x] = v
    end
    
    def each_index
      each_with_index do |line, y|
        line.each_with_index do |item, x|
          yield item,x,y
        end
      end
    end
  end
  
  class Surface
    def draw_string str, options = {}
      options[:font] ||= $font
      options[:color] ||= [255,255,255]
      options[:x] ||= 0
      options[:y] ||= 0

      options[:font].render(str.to_s, true, options[:color]).blit self, [options[:x], options[:y]]
    end
  end
  
end
